/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

使い方
    type    fixedAlphaTemp;
    refTemp 300;                    //参照温度（相手側の温度）
    alphaH  200;                    //熱伝達率
    KField_or_Kvalue  KField;       //熱伝導率をfieldから取得するか値で取得するか
    KField  lambda;                 //熱伝導率のfield名
    KValue  200;                    //熱伝導率の値（例では、KValueは使用せず）
    gradient uniform 0;             //gradientの
    value    uniform 300;           //値の初期値

\*---------------------------------------------------------------------------*/

#include "fixedAlphaTempFvPatchScalarField.H"
#include "dictionary.H"

//追加--------------
#include "mappedPatchBase.H"
#include "volMesh.H"
#include "addToRunTimeSelectionTable.H"
#include "fvPatchFieldMapper.H"
#include "volFields.H"
//----------------

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

//templateを削除し、「Foam::」追加・「<type>」を削除
//template<class Type>
Foam::fixedAlphaTempFvPatchScalarField::fixedAlphaTempFvPatchScalarField
(
    const fvPatch& p,
    //const DimensionedField<Type, volMesh>& iF
    const DimensionedField<scalar, volMesh>& iF
)
:
    //fvPatchField<Type>(p, iF),
    fixedGradientFvPatchScalarField(p, iF),
    //gradient_(p.size(), pTraits<Type>::zero)
    gradient_(p.size(), pTraits<scalar>::zero),
    //追加---------------------------------
    refTemp_(pTraits<scalar>::zero),
    alphaH_(pTraits<scalar>::zero),
    KName_("lambda"),
    KName_or_KValue_("KField-"),
    KValue_(pTraits<scalar>::zero)
    //-------------------------------------
{}


//template<class Type>
Foam::fixedAlphaTempFvPatchScalarField::fixedAlphaTempFvPatchScalarField
(
    //const fixedAlphaTempFvPatchScalarField<Type>& ptf,
    const fixedAlphaTempFvPatchScalarField& ptf,
    const fvPatch& p,
    //const DimensionedField<Type, volMesh>& iF,
    const DimensionedField<scalar, volMesh>& iF,
    const fvPatchFieldMapper& mapper
)
:
    //fvPatchField<Type>(ptf, p, iF, mapper),
    fixedGradientFvPatchScalarField(ptf, p, iF, mapper),
    gradient_(ptf.gradient_, mapper),
    //追加------------
    refTemp_(ptf.refTemp_),
    alphaH_(ptf.alphaH_),
    KName_(ptf.KName_),
    KName_or_KValue_(ptf.KName_or_KValue_),
    KValue_(ptf.KValue_)
    //----------------
{}


//template<class Type>
Foam::fixedAlphaTempFvPatchScalarField::fixedAlphaTempFvPatchScalarField
(
    const fvPatch& p,
    //const DimensionedField<Type, volMesh>& iF,
    const DimensionedField<scalar, volMesh>& iF,
    const dictionary& dict
)
:
    //fvPatchField<Type>(p, iF, dict),
    fixedGradientFvPatchScalarField(p, iF, dict),
    gradient_("gradient", dict, p.size()),
    //追加------------------------------
    refTemp_(pTraits<scalar>(dict.lookup("refTemp"))),
    alphaH_(pTraits<scalar>(dict.lookup("alphaH"))),
    KName_(dict.lookupOrDefault<word>("KField", "lambda")),
    KName_or_KValue_(dict.lookupOrDefault<word>("KField_or_KValue", "KField")),
    KValue_(pTraits<scalar>(dict.lookup("KValue")))
    //----------------------------------
{
    evaluate();
}


//template<class Type>
Foam::fixedAlphaTempFvPatchScalarField::fixedAlphaTempFvPatchScalarField
(
    //const fixedAlphaTempFvPatchScalarField<Type>& ptf
    const fixedAlphaTempFvPatchScalarField& ptf
)
:
    //fvPatchField<Type>(ptf),
    fixedGradientFvPatchScalarField(ptf),
    gradient_(ptf.gradient_),
    //追加----------------------
    refTemp_(ptf.refTemp_),
    alphaH_(ptf.alphaH_),
    KName_(ptf.KName_),
    KName_or_KValue_(ptf.KName_or_KValue_),
    KValue_(ptf.KValue_)
    //--------------------------
{}


//template<class Type>
Foam::fixedAlphaTempFvPatchScalarField::fixedAlphaTempFvPatchScalarField
(
    //const fixedAlphaTempFvPatchScalarField<Type>& ptf,
    const fixedAlphaTempFvPatchScalarField& ptf,
    //const DimensionedField<Type, volMesh>& iF
    const DimensionedField<scalar, volMesh>& iF
)
:
    //fvPatchField<Type>(ptf, iF),
    fixedGradientFvPatchScalarField(ptf, iF),
    gradient_(ptf.gradient_),
    //追加----------------------
    refTemp_(ptf.refTemp_),
    alphaH_(ptf.alphaH_),
    KName_(ptf.KName_),
    KName_or_KValue_(ptf.KName_or_KValue_),
    KValue_(ptf.KValue_)
    //--------------------------
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

//template<class Type>
void Foam::fixedAlphaTempFvPatchScalarField::autoMap
(
    const fvPatchFieldMapper& m
)
{
    //fvPatchField<Type>::autoMap(m);
    fixedGradientFvPatchScalarField::autoMap(m);
    gradient_.autoMap(m);
}


//template<class Type>
void Foam::fixedAlphaTempFvPatchScalarField::rmap
(
    //const fvPatchField<Type>& ptf,
    const fixedGradientFvPatchScalarField& ptf,
    const labelList& addr
)
{
    //fvPatchField<Type>::rmap(ptf, addr);
    fixedGradientFvPatchScalarField::rmap(ptf, addr);

    //const fixedAlphaTempFvPatchScalarField<Type>& fgptf =
    const fixedAlphaTempFvPatchScalarField& fgptf =
        //refCast<const fixedAlphaTempFvPatchScalarField<Type> >(ptf);
        refCast<const fixedAlphaTempFvPatchScalarField >(ptf);

    gradient_.rmap(fgptf.gradient_, addr);
}
//追加--------------------------------------
void Foam::fixedAlphaTempFvPatchScalarField::updateCoeffs()
{
    if (this->updated())
    {
        return;
    }

    //変数定義
    Field<scalar> flowHeat;
    //Field<scalar> intValue = this->patchInternalField();
    const fvPatchField<scalar>& patchValue = *this;
    if (KName_or_KValue_ == "KField")
    {
        //  Tの境界条件にlambda field の値を読んで設定する為、
        //　solver側のcreateFieldでfieldを定義する順番は、先にlambda、その後にTを定義する
        //　lambdaとTの定義する順番を間違えると「region0内にlambdaFieldが無い」のエラーが
        //　solver実行時に発生する。
        const fvPatchField<scalar>& lamp =
            patch().lookupPatchField<volScalarField, scalar>(KName_); 
        //flowHeat = (refTemp_ - intValue) * alphaH_ / lamp;
        flowHeat = (refTemp_ - patchValue) * alphaH_ / lamp;
    }
    else
    {
        //熱伝導率を値で取得する場合
        //flowHeat = (refTemp_ - intValue) * alphaH_ / KValue_;
        flowHeat = (refTemp_ - patchValue) * alphaH_ / KValue_;
    }

    gradient_ = flowHeat;

    fixedGradientFvPatchScalarField::updateCoeffs();
}
//------------------------------------------

//template<class Type>
void Foam::fixedAlphaTempFvPatchScalarField::evaluate(const Pstream::commsTypes)
{
    if (!this->updated())
    {
        this->updateCoeffs();
    }

    //Field<Type>::operator=
    Field<scalar>::operator=
    (
        this->patchInternalField() + gradient_/this->patch().deltaCoeffs()
    );

    //fvPatchField<Type>::evaluate();
    fixedGradientFvPatchScalarField::evaluate();
}


//template<class Type>
//tmp<Field<Type> > Foam::fixedAlphaTempFvPatchScalarField::valueInternalCoeffs
tmp<Field<scalar> > Foam::fixedAlphaTempFvPatchScalarField::valueInternalCoeffs
(
    const tmp<scalarField>&
) const
{
    //return tmp<Field<Type> >(new Field<Type>(this->size(), pTraits<Type>::one));
    return tmp<Field<scalar> >(new Field<scalar>(this->size(), pTraits<scalar>::one));
}


//template<class Type>
//tmp<Field<Type> > Foam::fixedAlphaTempFvPatchScalarField::valueBoundaryCoeffs
tmp<Field<scalar> > Foam::fixedAlphaTempFvPatchScalarField::valueBoundaryCoeffs
(
    const tmp<scalarField>&
) const
{
    return gradient()/this->patch().deltaCoeffs();
}


//template<class Type>
//tmp<Field<Type> > Foam::fixedAlphaTempFvPatchScalarField::
tmp<Field<scalar> > Foam::fixedAlphaTempFvPatchScalarField::
gradientInternalCoeffs() const
{
    //return tmp<Field<Type> >
    return tmp<Field<scalar> >
    (
        //new Field<Type>(this->size(), pTraits<Type>::zero)
        new Field<scalar>(this->size(), pTraits<scalar>::zero)
    );
}


//template<class Type>
//tmp<Field<Type> > Foam::fixedAlphaTempFvPatchScalarField::
tmp<Field<scalar> > Foam::fixedAlphaTempFvPatchScalarField::
gradientBoundaryCoeffs() const
{
    return gradient();
}


//template<class Type>
void Foam::fixedAlphaTempFvPatchScalarField::write(Ostream& os) const
{
    //fvPatchField<Type>::write(os);
    //gradient_.writeEntry("gradient", os);
    fixedGradientFvPatchScalarField::write(os);
    //追加-------------------------------------------------------------------
    os.writeKeyword("refTemp") << refTemp_ << token::END_STATEMENT << nl;
    os.writeKeyword("alphaH") << alphaH_ << token::END_STATEMENT << nl;
    os.writeKeyword("KField_or_KValue") << KName_or_KValue_ << token::END_STATEMENT << nl;
    os.writeKeyword("KField") << KName_ << token::END_STATEMENT << nl;
    os.writeKeyword("KValue") << KValue_ << token::END_STATEMENT << nl;
    //-----------------------------------------------------------------------
    this->writeEntry("value", os);
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

//追加-----------------------------------------------
namespace Foam
{
    makePatchTypeField
    (
        fvPatchScalarField,
        fixedAlphaTempFvPatchScalarField
    );
}
//---------------------------------------------------

// ************************************************************************* //
