/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::fixedAlphaTempFvPatchScalarField

Description
    Foam::fixedAlphaTempFvPatchScalarField

SourceFiles
    fixedAlphaTempFvPatchScalarField.C

\*---------------------------------------------------------------------------*/

#ifndef fixedAlphaTempFvPatchScalarField_H
#define fixedAlphaTempFvPatchScalarField_H

//#include "fvPatchFields.H"
//追加-----------------
#include "fixedGradientFvPatchFields.H"
//-----------------------

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
                           Class fixedGradientFvPatch Declaration
\*---------------------------------------------------------------------------*/

//template<class Type>
class fixedAlphaTempFvPatchScalarField
:
    //public fvPatchField<Type>
    public fixedGradientFvPatchScalarField
{
    // Private data

        //Field<Type> gradient_;
        scalarField gradient_;

        //追加----------------------
        //参照温度
        const scalar refTemp_;

        //熱伝達率
        scalar alphaH_;

        //熱伝導率のfield
        word KName_;

        //熱伝導率の取得方法
        word KName_or_KValue_;

        //熱伝導率の値
        scalar KValue_;
        //--------------------------


public:

    //- Runtime type information
    //TypeName("fixedGradient");
    TypeName("fixedAlphaTemp");


    // Constructors

        //- Construct from patch and internal field
        fixedAlphaTempFvPatchScalarField
        (
            const fvPatch&,
            //const DimensionedField<Type, volMesh>&
            const DimensionedField<scalar, volMesh>&
        );

        //- Construct from patch, internal field and dictionary
        fixedAlphaTempFvPatchScalarField
        (
            const fvPatch&,
            //const DimensionedField<Type, volMesh>&,
            const DimensionedField<scalar, volMesh>&,
            const dictionary&
        );

        //- Construct by mapping the given fixedAlphaTempFvPatchScalarField
        //  onto a new patch
        fixedAlphaTempFvPatchScalarField
        (
            //const fixedAlphaTempFvPatchScalarField<Type>&,
            const fixedAlphaTempFvPatchScalarField&,
            const fvPatch&,
            //const DimensionedField<Type, volMesh>&,
            const DimensionedField<scalar, volMesh>&,
            const fvPatchFieldMapper&
        );

        //- Construct as copy
        fixedAlphaTempFvPatchScalarField
        (
            //const fixedAlphaTempFvPatchScalarField<Type>&
            const fixedAlphaTempFvPatchScalarField&
        );

        //- Construct and return a clone
        //virtual tmp<fvPatchField<Type> > clone() const
        virtual tmp<fvPatchScalarField> clone() const
        {
            //return tmp<fvPatchField<Type> >
            return tmp<fvPatchScalarField>
            (
                //new fixedAlphaTempFvPatchScalarField<Type>(*this)
                new fixedAlphaTempFvPatchScalarField(*this)
            );
        }

        //- Construct as copy setting internal field reference
        fixedAlphaTempFvPatchScalarField
        (
            //const fixedAlphaTempFvPatchScalarField<Type>&,
            const fixedAlphaTempFvPatchScalarField&,
            //const DimensionedField<Type, volMesh>&
            const DimensionedField<scalar, volMesh>&
        );

        //- Construct and return a clone setting internal field reference
        //virtual tmp<fvPatchField<Type> > clone
        virtual tmp<fvPatchScalarField> clone
        (
            //const DimensionedField<Type, volMesh>& iF
            const DimensionedField<scalar, volMesh>& iF
        ) const
        {
            //return tmp<fvPatchField<Type> >
            return tmp<fvPatchScalarField>
            (
                //new fixedAlphaTempFvPatchScalarField<Type>(*this, iF)
                new fixedAlphaTempFvPatchScalarField(*this, iF)
            );
        }


    // Member functions

        // Return defining fields

            //- Return gradient at boundary
            //virtual Field<Type>& gradient()
            virtual Field<scalar>& gradient()
            {
                return gradient_;
            }

            //virtual const Field<Type>& gradient() const
            virtual const Field<scalar>& gradient() const
            {
                return gradient_;
            }


        // Mapping functions

            //- Map (and resize as needed) from self given a mapping object
            virtual void autoMap
            (
                const fvPatchFieldMapper&
            );

            //- Reverse map the given fvPatchField onto this fvPatchField
            virtual void rmap
            (
                //const fvPatchField<Type>&,
                const fixedGradientFvPatchScalarField&,
                const labelList&
            );


        // Evaluation functions

            //追加----------------------------------
            //- Update the coefficients associated whith the patch field
            virtual void updateCoeffs();
            //--------------------------------------

            //- Return gradient at boundary
            //virtual tmp<Field<Type> > snGrad() const
            virtual tmp<Field<scalar> > snGrad() const
            {
                return gradient_;
            }

            //- Evaluate the patch field
            virtual void evaluate
            (
                const Pstream::commsTypes commsType=Pstream::blocking
            );

            //- Return the matrix diagonal coefficients corresponding to the
            //  evaluation of the value of this patchField with given weights
            //virtual tmp<Field<Type> > valueInternalCoeffs
            virtual tmp<Field<scalar> > valueInternalCoeffs
            (
                const tmp<scalarField>&
            ) const;

            //- Return the matrix source coefficients corresponding to the
            //  evaluation of the value of this patchField with given weights
            //virtual tmp<Field<Type> > valueBoundaryCoeffs
            virtual tmp<Field<scalar> > valueBoundaryCoeffs
            (
                const tmp<scalarField>&
            ) const;

            //- Return the matrix diagonal coefficients corresponding to the
            //  evaluation of the gradient of this patchField
            //virtual tmp<Field<Type> > gradientInternalCoeffs() const;
            virtual tmp<Field<scalar> > gradientInternalCoeffs() const;

            //- Return the matrix source coefficients corresponding to the
            //  evaluation of the gradient of this patchField
            //virtual tmp<Field<Type> > gradientBoundaryCoeffs() const;
            virtual tmp<Field<scalar> > gradientBoundaryCoeffs() const;

        //- Write
        virtual void write(Ostream&) const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

//#ifdef NoRepository
//#   include "fixedAlphaTempFvPatchScalarField.C"
//#endif

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
